/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaq;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.ZwischenSpeicher;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Daten (Systemobjekdaten, ATG-Datenstze, Mengendaten) der Hierarchieobjekte 
 * einer Teilhierarchie vom Typ <code>HOHierarchie<7code> knnen zwischengespeichert und dann in
 * der ursprnglichen Hierarchie gelscht werden.<br>
 * Die zwischengespeicherten Daten knnen in neue Hierarcharchieobjekte der ursprnglichen Hierarchie
 * kopiert werden
 * <code>ZwischenSpeicherAQ</code>implementiert Spezialverahlten fr den TKA AQ
 * 
 * @author Sans
 */
public class ZwischenSpeicherAQ extends ZwischenSpeicher
{
 /**
  * Debug-Logger fr Logging-Ausgaben
  */
  private final static Debug _debug = Debug.getLogger();
	
	/**
	 * Zwischenspeicher fr den Inhalt des Referenz-Arrays Zeichen von Ligaturen
	 */
	private Map<String, List<String>> _ligaturenZeichen = new HashMap<String, List<String>>();
	
	/**
	 * Zwischenspeicher fr die Referenz Font von Wechseltexten
	 */
	private Map<String, String> _wechseltextFont = new HashMap<String, String>();
	
	/**
	 * Zwischenspeicher fr den Inhalt des Referenz-Arrays Textdefinition von Wechseltexten
	 */
	private Map<String, List<String>> _wechseltextTextdefinition = new HashMap<String, List<String>>(); 
	
	 /**
   * Erzeugt eine neue Instanz vom Typ <code>ZwischenSpeicherAQ<7code>
   * 
   * @param hierarchie zu bearbeitende Teilhierarchie, die eine Kopie einer ursprnglichen Hierarchie 
   * darstellt
   * @param hm Hierarchiemanager fr die ursprngliche Hierarchie
   */
	public ZwischenSpeicherAQ(HOHierarchie hierarchie, IHierarchieManager hm)
  {
    super(hierarchie, hm);
  }
	
	/**
   * Kopiert die zwischengespeicherten Daten (Systemobjektdaten, ATG-Daten, Mengendaten) in ein neues HO
   * Beim Lschen der untergeordnetenen Objekte des HOT Zeichensatz werden zuerst die Ligaturen, dann 
   * die Zeichen gelscht
   * 
   * @param ho das neue HO
   * @param zwischengespeichertesHO das HO, fr das in der Teilhierarchie Daten zwischengespeichert wurden
   * @return den Status der Aktion
   */
	@Override
	public IStatus speichernUndLoeschenRekursiv(HierarchieObjekt ho)
  {
    MultiStatus status = new MultiStatus(Activator
        .PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
        " vom Typ " + ho.getHierarchieObjektTyp(), null);
    
    IHierarchie hierarchie = _hm.getHierarchie(); 
    
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    if (HOT_AQ.Zeichensatz.equals(hot))
    {
    	// Sonderbehandlung AQ: Zuerst alle Ligaturen lschen
    	List<HierarchieObjekt> untergeordneteObjekte = 
    		new LinkedList<HierarchieObjekt>(hierarchie.getUntergeordneteObjekte(ho));
    	for (HierarchieObjekt kind: untergeordneteObjekte)
    	{
    		if (HOT_AQ.Ligaturen.equals(kind.getHierarchieObjektTyp()))
    		{
    			status.add(speichernUndLoeschenRekursiv(kind));
  	      if (status.matches(IStatus.CANCEL | IStatus.ERROR))
  	      {
  	        break;
  	      }
    		}
    	}
    }
    
    if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
    { 
	    while (!hierarchie.getUntergeordneteObjekte(ho).isEmpty())
	    {
	      HierarchieObjekt kind = hierarchie.getUntergeordneteObjekte(ho).get(0);
	      
	      status.add(speichernUndLoeschenRekursiv(kind));
	      
	      if (status.matches(IStatus.CANCEL | IStatus.ERROR))
	      {
	        break;
	      }
	    }
	    
	    erzeugeZwischengespeicherteDaten(ho);
	    
	    if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
	    {  
	      status.merge(_hm.loescheHO(ho));
	    }
	  }
    
    return status;
  }
	
	 /**
   * Erzeugen und speichern der zwischengespeicherten Daten
   * 
   * @param ho das HO in der Teilhierarchie, fr das Daten zwischengespeichert werden 
   */
  protected void erzeugeZwischengespeicherteDaten(HierarchieObjekt ho)
  {
  	super.erzeugeZwischengespeicherteDaten(ho);
  	
  	if (HOT_AQ.Ligaturen.equals(ho.getHierarchieObjektTyp()))
  	{
  		SystemObject so = ho.getSystemObjekt(0);
  		ConfigDataModel dataModel = _hm.getHierarchie().getDataModel(); 
  		AttributeGroup atg = dataModel.getAttributeGroup("atg.ligaturen");
  		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  		Data data = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
  		
  		_ligaturenZeichen.remove(so.getPid());
  		
  		if (data != null) 
  		{
  			List<String> list = new LinkedList<String>();
  			ReferenceArray refArray = data.getReferenceArray("Zeichen");
  			for (int i = 0; i < refArray.getLength(); i += 1)
  			{
  				String pid = refArray.getReferenceValue(i).getSystemObjectPid();
  				if (pid != null && !pid.isEmpty())
  				{	
  					list.add(pid);
  				}	
  			}
  			
  			if (!list.isEmpty())
  			{	
  				_ligaturenZeichen.put(so.getPid(), list);
  			}	
  		}
  	}
  	else if (HOT_AQ.WechselText.equals(ho.getHierarchieObjektTyp()))
  	{
  		SystemObject so = ho.getSystemObjekt(0);
  		ConfigDataModel dataModel = _hm.getHierarchie().getDataModel(); 
  		AttributeGroup atg = dataModel.getAttributeGroup("atg.wechseltext");
  		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  		Data data = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
  		
  		_wechseltextFont.remove(so.getPid());
  		_wechseltextTextdefinition.remove(so.getPid());
  		
  		if (data != null) 
  		{
  			List<String> list = new LinkedList<String>();
  			
  			String font = data.getReferenceValue("Font").getSystemObjectPid();
  			
  			if (font != null && !font.isEmpty())
  			{
  				_wechseltextFont.put(so.getPid(), font);
  			}
  			
  			ReferenceArray refArray = data.getReferenceArray("Textdefinition");
  			for (int i = 0; i < refArray.getLength(); i += 1)
  			{
  				String pid = refArray.getReferenceValue(i).getSystemObjectPid(); 
  				if (pid != null && !pid.isEmpty())
  				{	
  					list.add(pid);
  				}
  			}
  			
  			if (!list.isEmpty())
  			{	
  				_wechseltextTextdefinition.put(so.getPid(), list);
  			}
  		}
  	}
  }
	
  /**
   * Kopiert die zwischengespeicherten Daten (Systemobjektdaten, ATG-Daten, Mengendaten) in ein neues HO
   * 
   * @param ho das neue HO
   * @param zwischengespeichertesHO das HO, fr das in der Teilhierarchie Daten zwischengespeichert wurden
   * @return den Status der Aktion
   */
  @Override
  protected IStatus kopiereZwischengespeichertDaten(HierarchieObjekt ho, HierarchieObjekt zwischengespeichertesHO)
  {
    HODaten hoDaten = _hoDaten.get(zwischengespeichertesHO);
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Hierarchieobjekt " +
        zwischengespeichertesHO, null);
    
    if (hoDaten != null)
    {
      int index = 0;
      
      for (SystemObject lauf: hoDaten.getSystemObjekte())
      {
        SystemObjektDaten soDaten = hoDaten.getSystemObjektDaten(lauf);
        Assert.isTrue(soDaten != null);
        SystemObject so = ho.getSystemObjekt(index);
        Assert.isTrue(so.getPid().equals(soDaten.getPid()));
        ConfigDataModel dataModel = _hm.getHierarchie().getDataModel();
        MultiStatus unterStatus = status;
        
        if (hoDaten.getSystemObjekte().size() > 0)
        {
          unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten fr Systemobjekt " +
              soDaten.getPid(), null);
        }
        
        try
        {
          so.setName(soDaten.getName());
          
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Setze Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid()));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Name \"" + soDaten.getName() + "\" fr Systemobjekt "
              + so.getPid() + " konnte nicht gesetzt werden", e));
        }
        
        for (String atgName: soDaten.getAtg())
        {
          AttributeGroup atg = dataModel.getAttributeGroup(atgName);
          if (atg != null)
          {  
            Data data = soDaten.getAtgDaten(atgName);
            
            if (HOT_AQ.Ligaturen.equals(hot) && "atg.ligaturen".equals(atgName))
            {
            	ReferenceArray zeichen = data.getReferenceArray("Zeichen");
            	List<String> liste = _ligaturenZeichen.get(soDaten.getPid());
            	if (liste != null && !liste.isEmpty())
            	{
            		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
            		zeichen.setLength(liste.size());
            		for (int i = 0; i < zeichen.getLength(); i += 1)
            		{
            			zeichen.getReferenceValue(i).setSystemObjectPid(liste.get(i), lookup);
            		}
            	}
            	else
            	{
            		zeichen.setLength(0);
            	}
            }
            else if (HOT_AQ.WechselText.equals(hot) && "atg.wechseltext".equals(atgName))
            {
            	ReferenceValue font = data.getReferenceValue("Font");
            	String pid = _wechseltextFont.get(soDaten.getPid());
            	if (pid != null && !pid.isEmpty())
            	{
            		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
            		font.setSystemObjectPid(pid, lookup);
            	}
            	else
            	{
            		font.setSystemObject(null);
            	}
            	
            	ReferenceArray textdefinition = data.getReferenceArray("Textdefinition");
            	List<String> liste = _wechseltextTextdefinition.get(soDaten.getPid());
            	if (liste != null && !liste.isEmpty())
            	{
            		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
            		textdefinition.setLength(liste.size());
            		for (int i = 0; i < textdefinition.getLength(); i += 1)
            		{
            			textdefinition.getReferenceValue(i).setSystemObjectPid(liste.get(i), lookup);
            		}
            	}
            	else
            	{
            		textdefinition.setLength(0);
            	}
            }
            
            try
            {
              so.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid()));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                  + atg.getPid() + " fr Systemobjekt " + so.getPid() + " konnten nicht kopiert werden", e));
            }
          }
          else
          {
            _debug.warning("Attributgruppe " + atgName + " ist nicht vorhanden");
          }
        }
        
        for (String mengenName: soDaten.getDynMengen())
        {
          ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
          if (menge != null)
          {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
            if (atg != null)
            {  
              Data data = soDaten.getDynMengenDaten(mengenName);
              try
              {
                menge.setConfigurationData(atg, data);
                status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid()));
              }
              catch (ConfigurationChangeException e)
              {
                status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG "
                    + atg.getPid() + " fr die Menge "  +  mengenName + " am Systemobjekt " + so.getPid(), e));
              }
            }
            else
            {
              _debug.warning("Attributgruppe atg.dynamischeMenge ist nicht vorhanden");
            }
          }
          else
          {
            status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                "\" ist am Systemobjekt " + so.getPid() + " nicht vorhanden"));
          }
        }
        
        for (String mengenName: soDaten.getMengen())
        {
          List<SystemObject> elementListe = soDaten.getMengenElemente(mengenName);
          if (elementListe != null && elementListe.size() > 0)
          {
            ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
            if (menge != null)
            {
              for (SystemObject element: elementListe)
              {
                try
                {
                  menge.add(element);
                  status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + " hinzugefgt"));
                }
                catch (ConfigurationChangeException e)
                {
                  status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + element.getPid() +
                      " konnte der  Menge "  +  mengenName + " am Systemobjekt " + so.getPid() + 
                      " nicht hinzugefgt werden ", e));
                }
              }
            }
            else
            {
              status.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "MengeName \"" + mengenName + 
                  "\" ist am Systemobjekt " + so.getPid() + " nicht vorhanden"));
            }
          }
        }
        
        index += 1;
        if (unterStatus != status)
        {
          status.add(unterStatus);
        }
      }
    }
    else
    {
      _debug.warning("Fr Hierarchieobjekt " + ho + " wurden keine zwischengespeicherte Daten gefunden");
    }
    
    return status;
  }
  
	/**
   * Kopiert die zwischengespeicherten Daten gemss der bergebenen Teilhierarchie in neue Hierarchieobjekte
   * der ursprnglichen Hierarchie
   * 
   * @param ho das zu kopierende HO in der Teilhierarchie
   * @param srcKB der (Hierarchie-) KB, aus dem kopiert wird
   * @param destKB der (Hierarchie-) KB, in dem kopiert wird
   * @param vater neues Vater-Objekt, unter dem das zu erstellende neue HO angelegt wird
   * @param isWurzel bestimmt, ob die Wurzel oder eine untergeordnetes Objekt der Teilhierarchie kopiert wird
   * @return den Status der Aktion
   */
	@Override
  protected HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea srcKB, ConfigurationArea destKB, 
      Object vater, boolean isWurzel)
  {
    MultiStatus meldung;
    
    if (ho == _hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    ConfigurationArea kb = ho.getSystemObjekt(0).getConfigurationArea();
    if (kb == srcKB)
    {
      kb = destKB;
    }
    
    String[] pid = new String[ho.getSystemObjekte().size()];
    for (int i = 0; i < pid.length; i += 1)
    {
      pid[i] = ho.getSystemObjekt(i).getPid();
    }
    
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    
    HOMitStatus result = _hm.erzeugeNeuesHO(kb, vater, pid, hot, false);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();
    
    if (neu != null)
    {
      if (!hot.isSystemObjektVorhanden())
      { 
        meldung.merge(kopiereZwischengespeichertDaten(neu, ho));
      } 
      
      IHierarchieObjektTyp neuHot = neu.getHierarchieObjektTyp();

      // Sonderbehandlung AQ: Wenn neuHot == AnzeigeTyp zuerst alle Zeichenstze kopieren
      for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte( ho ))
      {
      	if (HOT_AQ.AnzeigeTyp.equals(neuHot) && HOT_AQ.Zeichensatz.equals(kind.getHierarchieObjektTyp()))
  			{
      		 result = kopiereHORekursiv(kind, srcKB, destKB, neu, false);
           meldung.add( result.getStatus() );
  			}
      }
      
      for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte( ho ))
      {
      	if (HOT_AQ.AnzeigeTyp.equals(neuHot) && HOT_AQ.Zeichensatz.equals(kind.getHierarchieObjektTyp()))
  			{
      		// wurde schon oben kopiert
      		continue;
  			}
        result = kopiereHORekursiv(kind, srcKB, destKB, neu, false);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
}
