/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Gemeinsam genutzte Funktionalitt HierarchieManager
 * 
 * @author Sans
 *
 */
public abstract class AbstractHierarchieManager
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /*
   * ************************ Zuordnen Hierarchie
   */
  
  private IHierarchie _hierarchie;

  public IHierarchie getHierarchie()
  {
    return _hierarchie;
  }

  public void setHierarchie(IHierarchie hierarchie)
  {
    _hierarchie = hierarchie;
  }
  
  /*
   * ************************ Ende zuordnen Hierarchie
   */
  
  
  /*
   * ************************ Austauschen Systemobjekt im Hierarchieobjekt 
   */
  
  protected ConfigurationObject dupliziereObjekt(ConfigurationObject ko, ConfigurationArea kb, String pid)
    throws ConfigurationChangeException
  {
    Assert.isLegal(ko != null);
    
    String neuePid = pid != null ? pid : ko.getPid();
    ConfigurationArea neuerKb = kb != null ? kb : ko.getConfigurationArea(); 
    
    List<ObjectSet> sets = new Vector<ObjectSet>();
    ConfigurationObject neu = null;
    ConfigurationObject result = null;
    
    try
    {
      for (ObjectSet set : ko.getObjectSets())
      {
        ObjectSetType setTyp = set.getObjectSetType();
        Assert.isTrue(setTyp.getReferenceType() != ReferenceType.COMPOSITION);
        ObjectSet menge = (ObjectSet)neuerKb.createConfigurationObject(set.getObjectSetType(), "",
            set.getName(), null);
        List<SystemObject> elemente = set instanceof NonMutableSet ? ((NonMutableSet)set)
            .getElementsInModifiableVersion() : set.getElements();
    //    menge.add(elemente.toArray(new SystemObject[elemente.size()]));
        for (SystemObject element: elemente)
        {
          if (element != null)
          {
            menge.add(element);
          }
          else
          {
            _debug.error("Nullreferenz gefunden in Menge " + menge.getName() + 
                " des Objekts " + ko.getPid());
          }
        }
            
        sets.add(menge);
      }
    
      neu = neuerKb.createConfigurationObject((ConfigurationObjectType)ko.getType(),
          neuePid == null ? ko.getPid() : neuePid, ko.getName(), sets);
    
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)ko.getDataModel());
    
      for (AttributeGroupUsage atgu : ko.getUsedAttributeGroupUsages())
      {
        Data dataSrc = ((ConfigSystemObject)ko).getConfigurationData(atgu, lookup);
        if (dataSrc != null)
        {
          Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
          neu.setConfigurationData(atgu, dataDest);
        }
      }
      result = neu;
    }
    finally
    {
      // Rollback
      if (result == null)
      {
        for (ObjectSet set : sets)
        {
          set.invalidate();
        }
        if (neu != null)
        {
          neu.invalidate();
        }
      }
    }
    
    return result;
  }
  
  protected abstract void bereinigeSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject koAlt,
      ConfigurationObject koNeu) throws ConfigurationChangeException;
  
  protected ConfigurationObject behandleSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject ko, 
      String neuePid) throws ConfigurationChangeException
  {
    return behandleSystemObjektAenderung(ho, ko, null, neuePid, false);
  }
  
  protected ConfigurationObject behandleSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject ko, 
      ConfigurationArea kb, String neuePid, boolean loescheVorab) throws ConfigurationChangeException
  {
    if (loescheVorab)
    {
      bereinigeSystemObjektAenderung(ho, ko, null);
      ko.invalidate();
    }
    
    ConfigurationObject result = dupliziereObjekt(ko, kb, neuePid);
    
    if (loescheVorab)
    {
      bereinigeSystemObjektAenderung(ho, null, result);
    }
    else
    {
      bereinigeSystemObjektAenderung(ho, ko, result);
      ko.invalidate();
    }
    
    return result;
  }
  
  /*
   * ************************ Ende austauschen Systemobjekt im Hierarchieobjekt 
   */
  
  
  /*
   * ************************ Zuordnenen neuer Konfigurationsbereich zu Systemobjekt
   */
  
  private boolean zuordnenKBRekursiv(HierarchieObjekt ho, Object neuerVater, ConfigurationArea kbAlt, 
      ConfigurationArea kbNeu, MultiStatus status)
  {
    boolean result = zuordnenKB(ho, neuerVater, kbAlt, kbNeu, status);
    
    for (HierarchieObjekt kind: getHierarchie().getUntergeordneteObjekte(ho))
    { 
      zuordnenKBRekursiv(kind, kbAlt, kbNeu, status);
    } 
    
    return result;
  }  
    
  protected boolean zuordnenKBRekursiv(HierarchieObjekt ho, ConfigurationArea kbAlt, ConfigurationArea kbNeu, 
      MultiStatus status)
  {
    boolean result = zuordnenKB(ho, null, kbAlt, kbNeu, status);
    
    if (result)
    {
      for (HierarchieObjekt kind: getHierarchie().getUntergeordneteObjekte(ho))
      { 
        zuordnenKBRekursiv(kind, kbAlt, kbNeu, status);
      } 
    }
    
    return result;
  }
  
  private boolean zuordnenKB(HierarchieObjekt ho, Object neuerVater, ConfigurationArea kbAlt, ConfigurationArea kbNeu,
      MultiStatus status)
  {
    if (kbAlt.equals(kbNeu))
    {
      return false;
    }
    
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    
    if (hot.isSystemObjektVorhanden())
    {
      return true;
    }
    
    if (ho.getSystemObjekt(0).getConfigurationArea().equals(kbAlt) || kbNeu.equals(neuerVater))
    {
      return zuordnenKB(ho, kbNeu, status);
    }
    
    return true;
  }
  
  protected boolean zuordnenKB(HierarchieObjekt ho, ConfigurationArea kb, MultiStatus status)
  {
    int index = 0;
    boolean result = false;
    
    for (SystemObject so: ho.getSystemObjekte())
    {
      if (so.getConfigurationArea().equals(kb))
      {
        result = true;
        continue;
      }
      
      ConfigurationObject ko = (ConfigurationObject)so;
      boolean aktiviert = ko.getValidSince() < ko.getConfigurationArea().getModifiableVersion();
      
      try
      {
        ConfigurationObject neu = behandleSystemObjektAenderung(ho, ko, kb, null, !aktiviert);
        ho.setSystemObjekt(index, neu);
        result = true;
        
        status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Fr Hierarchieobjekt " + ho + 
            " Objekt mit PID " + so.getPid() + " gelscht und im Konfigurationsbereich " + 
            kb.getPid() + " identisch neu erzeugt " + so.getType().getPid() + " erzeugt"));
        
        index += 1;
      }
      catch (Exception e)
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Beim Lschen oder Neuerzeugen von Objekt mit Pid " + 
            so.getPid() + " bei Hierarchieobjekt" + ho + " ist eine Ausnahme aufgetreten", e));
        if (!ko.isValid())
        {
          try
          {
            ko = behandleSystemObjektAenderung(ho, ko, null);
            ho.setSystemObjekt(index, ko);
          } 
          catch(Exception ex)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Weitere Fehlerbehandlung nicht mglich", e));
          }
        }
      }
    }
    
    return result;
  }
  
  /*
   * ************************ Ende zuordnenen neuer Konfigurationsbereich zu Systemobjekt
   */
  
  
  /*
   * ************************ HO verschieben
   */
  protected abstract void behandleVerschieben(HierarchieObjekt ho, Object neuerVater, 
      MultiStatus status);
  
  protected abstract void behandleVerschiebenInAnderenKB(HierarchieObjekt ho, Object neuerVater, 
      ConfigurationArea srcKB, ConfigurationArea desKB, MultiStatus status);
  
  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
  {
    IHierarchie hierarchie = getHierarchie();
    Object alterVater = hierarchie.getUebergeordnetesObjekt(ho);
    ConfigurationArea srcKB = hierarchie.getKonfigurationsBereich(alterVater);
    Assert.isNotNull(srcKB, "Der Konfigurationsbereich zu " + alterVater + " darf nicht null sein");
    ConfigurationArea destKB = hierarchie.getKonfigurationsBereich(neuerVater);
    Assert.isNotNull(destKB, "Der Konfigurationsbereich zu " + neuerVater + " darf nicht null sein");

    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
    
    HierarchieObjekt neu = null;

    if (srcKB.equals(destKB)) 
    {
      // HO wird im gleichen kb verschoben
      if (neuerVater.equals(alterVater))
      {
        status.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Objekt " + ho
            + " kann nicht in sich selbst verschoben werden"));
      }
      else
      {
        behandleVerschieben(ho, neuerVater, status);
        
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL ))
        {
          neu = ho; // HO verschoben
        }
      }
    }
    else
    {
       // HO wird in einen anderen kb verschoben
       if (zuordnenKBRekursiv(ho, neuerVater, srcKB, destKB, status))
       {
         behandleVerschiebenInAnderenKB(ho, neuerVater, srcKB, destKB, status);
         neu = ho; // HO verschoben
       }
  
    }
    return new HOMitStatus( neu, status );
  }
  
  /*
   * ************************ Ende HO verschieben
   */
}
