/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.List;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Die Schnittstelle IHierarchie modelliert die Hierarchie von Instanzen vom Typ Hierarchieobjekt.
 * Einem Hierarchieobjekt kann dabei ein Objekt von beliebigem Typ (Object) bergeordnet sein,
 * insbesondere auch ein Objekt vom Typ ConfigurationArea.
 * 
 * @author Sans
 *
 */
public interface IHierarchie
{
  /**
   * Bestimmt das Datenmodell zum Zugriff auf die Konfiguration
   * 
   * @return das Datenmodell zum Zugriff auf die Konfiguration
   */
  public ConfigDataModel getDataModel();

//  /**
//   * Bestimmt die Wurzelobjekte - die Objekte auf der obersten Ebene - dieser Hierarchie
//   * 
//   * @return Eine Liste der Objekte auf der obersten Ebene Hierarchie
//   */
//  public List getWurzelObjekte();
  
  /**
   * Bestimmt die untergeordneten Hierarchieobjekte zu einem Objekt der Hierarchie
   * 
   * @param objekt das Objekt, zu dem die untergordenten Objekte bestimmt werden sollen
   * @return Liste der untergeordneten Objekte zum bergebenen Objekt
   */
  public List<HierarchieObjekt> getUntergeordneteObjekte(Object objekt);
  
  /**
   * Bestimmt das bergeordnete Objekt zu einem Hierachie
   * 
   * @param ho das Hierarchieobjekt, zu dem das bergeordnete Objekt bestimmt werden soll
   * @return das bergeordnete Objekt zum bergebenen Hierarchieobjekt
   */
  public Object getUebergeordnetesObjekt(HierarchieObjekt ho);
  
  /**
   * Fgt ein Hierarchieobjekt in die Hierarchie ein
   * 
   * @param vater das Eltern-Objekt
   * @param kind das Hierachieobjekt, das unter vater eingefgt werden soll
   */
  public void objektHinzufuegen(Object vater, HierarchieObjekt kind);
  
  /**
   * Entfernt ein Hierarchieobjekt aus der Hierarchie
   * 
   * @param vater das Eltern-Objekt
   * @param kind das Hierachieobjekt, das unter vater entfernt werden soll
   */
  public void objektEntfernen(Object vater, HierarchieObjekt kind);
  
  /**
   * Bestimmt die Anzahl der Hierarchieebenen
   * 
   * @return die Anzahl der Hierarchieebenen
   */
  public int getAnzahlEbenen();
  
  /**
   * Bestimmt, ob alle Systemobjekte eines Hierarchieobjekts aktiviert sind (ggf. auch fr alle 
   * untergeordneten Hierarchieobjekte)
   * 
   * @param ho das Hierarchieobjekt, dessen Systemobjekte geprft werden sollen
   * @param rekursiv Flag, das angibt, ob auch die Systemobjekte der untergeordneten 
   * Hierarchieobjekte - rekursiv - berprft werden sollen
   * @return <code>true</code>, wenn alle Systemobjekte aktiviert sind <code>false</code> sonst
   */
  public boolean alleObjekteAktiviert(HierarchieObjekt ho, boolean rekursiv);
  
  /**
   * Bestimmt den Konfigurationsbereich fr ein Objekt der Hierarchie
   * 
   * @param objekt das Obejkt in der Hierarchie, fr das der KB bestimmt werden soll
   * @return den KB des bergebenen Objekts oder <code>null</code>, falls das Objekt nicht zur
   * Hierarchie gehrt
   */
  public ConfigurationArea getKonfigurationsBereich(Object objekt);
  
   /**
   * Verschiebt ein Hierarchieobjekt der Hierarchie von einem bergeordneten Objekt
   * zu einem anderen bergeordneten Objekt
   * 
   * @param alterVater altes bergeordnetes Objekt
   * @param neuerVater neues bergeordnetes Objekt
   * @param kind zu verschiebendes Hierarchieobjekt
   */
  public void objektVerschieben(Object alterVater, Object neuerVater, HierarchieObjekt kind);
  
  /**
   * Meldet eine IKBHierarchieBeobachter-Instanz auf nderungen an 
   * 
   * @param beobachter anzumeldende IKBHierarchieBeobachter-Instanz
   */  
  public void beobachterAnmelden(IHierarchieBeobachter beobachter);
      
  /**
  * Meldet eine angemeldete IKBHierarchieBeobachter-Instanz ab
  * 
  * @param beobachter abzumeldende IKBHierarchieBeobachter-Instanz
  */
  public void beobachterAbmelden(IHierarchieBeobachter beobachter);
  
  /**
   * Informiert ggf. angemeldete Beobachter ber die Aktualisierung eines Hierarchieobjekts
   * 
   * @param ho das aktualisierte Hierarchieobjekt
   */
  public void notifyHOGeaendert(HierarchieObjekt ho);

}
