/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.pruefungen;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

/**
 * PruefungenTools stellt von allen Prfmodulen (und anderen) bentigte Methoden zur Verfgung
 * 
 * @author Sans
 *
 */
public class PruefungenTools
{
  /**
   * Code fr Statusmeldung, der doppelte Pid anzeigt
   */
  public final static int CODE_DOPPELTE_PID = 0x00001;
  
  /**
   * Das zugrundeliegende Datenmodell der Konfiguration
   */
  ConfigDataModel _dataModel;
  
  /**
   * Erzeugt eine Instanz vom Typ PruefungenTools fr ein bestimmtes Konfigurations-Datenmodell
   * 
   * @param dataModel das zugrundeliegende Datenmodell der Konfiguration
   */
  public PruefungenTools(ConfigDataModel dataModel)
  {
    _dataModel = dataModel;
  }

  /**
   * Prft fr eine PID, ob diese zulssig ist, d. h. ob diese innerhalb der Konfiguration 
   * noch nicht vergeben ist.
   * <br>Wenn eine (oder mehrere) vorhandene Pid gefunden werden, wird in den entsprecheneden
   * Status-Meldungen das Code-Bit CODE_DOPPELTE PID gesetzt.
   * 
   * @param pid die zu berprfende PID
   * @return Status, der das Ergebnis der PID-Prfung beinhaltet
   */
  public IStatus pruefePid(String pid)
  {
    return pruefePid(pid, null);
  }

  
  /**
   * Prft fr eine PID, ob diese zulssig ist, d. h. ob diese innerhalb der Konfiguration 
   * eindeutig ist.
   * <br>Wenn eine (oder mehrere) vorhandene Pid gefunden werden, wird in den entsprecheneden
   * Status-Meldungen das Code-Bit CODE_DOPPELTE PID gesetzt.
   * 
   * @param pid die zu berprfende PID
   * @param so das Systemobjekt, fr das diese PID geprft werden soll. Wird hier <code>null</code>
   * bergeben, so wird geprft, ob ein neues Systemobjekt mit der bergebenen PID angelegt werden 
   * kann
   * @return Status, der das Ergebnis der PID-Prfung beinhaltet
   */
  public IStatus pruefePid(String pid, SystemObject so)
  {
    return pruefePid(pid, so, null);
  }
  
  /**
   * Prft fr eine PID, ob diese zulssig ist, d. h. ob diese innerhalb der Konfiguration 
   * (oder auch nur innerhalb eines Konfigurationsbereiches) eindeutig ist.
   * <br>Wenn eine (oder mehrere) vorhandene Pid gefunden werden, wird in den entsprecheneden
   * Status-Meldungen das Code-Bit CODE_DOPPELTE PID gesetzt.
   * 
   * @param pid die zu berprfende PID
   * @param so das Systemobjekt, fr das diese PID geprft werden soll. Wird hier <code>null</code>
   * bergeben, so wird geprft, ob ein neues Systemobjekt mit der bergebenen PID angelegt werden 
   * kann
   * @param kb der Konfigurationsbereich, auf den sich die Eindeutigkeitsprfung bezieht. Wird hier 
   * <code>null</code> bergeben, so bezieht sich die Prfung auf die gesamte Konfiguration
   * @return Status, der das Ergebnis der PID-Prfung beinhaltet
   */
  public IStatus pruefePid(String pid, SystemObject so, ConfigurationArea kb)
  {
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe PID " + pid, null);
    
    if (kb != null)
    {
      for (SystemObject lauf: kb.getCurrentObjects())
      {
        if ((lauf instanceof ConfigurationObject &&
            !(((ConfigurationObject)lauf).getNotValidSince() >= kb.getModifiableVersion())) ||
            (lauf instanceof DynamicObject &&
                !(((DynamicObject)lauf).getNotValidSince() >= kb.getModifiableVersion()))        )
        { 
          // Objekt wird in zuknftiger Version nicht ungltig
          pruefePid(lauf, so, pid, status);
        } 
      }
      for (SystemObject lauf: kb.getNewObjects())
      {
        pruefePid(lauf, so, pid, status);
      }
    }
    else
    {
      for (ConfigurationArea lauf: _dataModel.getAllConfigurationAreas().values())
      {
        status.merge(pruefePid(pid, so, lauf));
      }
    }
    
    return status;
  }
    
  /**
   * Prft fr ein Systemobjekt, ob es die bergebene PID hat. Wenn dies zutrifft und
   * das Systemobjekt ungleich einem bergebenen vorhandenem Objekt ist, wird die
   * bergebene Meldung um einen entsprechenden Status mit entsprechendem Grad (wegen 
   * mehrfache PID) erweitert.
   * <br>Wenn eine (oder mehrere) vorhandene Pid gefunden werden, wird in den entsprecheneden
   * Status-Meldungen das Code-Bit CODE_DOPPELTE PID gesetzt.
   * 
   * @param gefundenesObjekt das Objekt, das geprft werden soll
   * @param vorhandenesObjekt ein Objekt, fr das keine Statuserweiterung bei positiver
   * Prfung erfolgen soll
   * @param pid die zu prfenden PID
   * @param status die Status-Multimeldung, die ggf. zu erweitern ist
   */
  private void pruefePid(SystemObject gefundenesObjekt, 
      SystemObject vorhandenesObjekt, String pid, MultiStatus status)
  {
    if (pid.equals(gefundenesObjekt.getPid()))
    {
      if (!gefundenesObjekt.equals(vorhandenesObjekt))
      {
//        int severity = vorhandenesObjekt.getConfigurationArea().equals(gefundenesObjekt.getConfigurationArea())?
//            Status.ERROR : Status.WARNING;
        
        status.add(new Status(/* severity */Status.ERROR, Activator.PLUGIN_ID, CODE_DOPPELTE_PID,
          "PID " + pid + " wird bereits verwendet von Objekt vom Typ " + gefundenesObjekt.getType().getPid() +
          " im KB " + gefundenesObjekt.getConfigurationArea().getPid(), null));  
      }
    }
  }
  
  /**
   * Prft den Wert eines Attributes eines Datensatzes, ob dieser<br>
   * 1. Korrekt definiert ist<br>
   * 2. Bei Textattributen eine ggf. vorhandene Lngenbeschrnnkung<br>
   * 3. Optional, ob der Wert des Attribut nicht leer ist, falls es als Pflichtfeld 
   * ausgewiesen ist<br>
   * Die Prfungen erfolgen rekursiv fr die nicht-atomaren Attributtypen Array 
   * und Attributliste
   * 
   * @param attInfo Attribut-Info des zu prfenden Attributs
   * @param daten Datensatz, der den zu prfenden Werte enthlt
   * @param ignoreIsPflichtFeld wenn <code>true</code> finden keine Prfungen auf 
   * Pflichtfeldeigenschaften statt, wenn <code>false</code> werden die 
   * Pflichtfeldeigenschaften gemss Attributinfo geprft
   * @return Status, der das Ergebnis der Prfung enthlt
   */
  public static IStatus pruefeDatenDefinition(IAttributInfo attInfo, Data daten, 
      boolean ignoreIsPflichtFeld)
  {
    MultiStatus status = null;
    Data item = daten.getItem(attInfo.getName());
    
    if (!attInfo.isArray())
    {
      if (!attInfo.hasInnereAttribute())
      { 
        Assert.isTrue(item.isPlain(), "An dieser Stelle muss der bergebene Datensatz fr \"" +
            attInfo.getName() + "\" ein atomares Datum liefern");
        status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
            attInfo.getName(), null);
        pruefeDatenDefinition(item, ignoreIsPflichtFeld ? false : attInfo.isPflichtFeld(), status);
      }
      else
      {
        Assert.isTrue(item.isList(), "An dieser Stelle muss der bergebene Datensatz fr \"" +
            attInfo.getName() + "\" eine Attributliste liefern");
        status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Bearbeite Attributliste " +
            attInfo.getName(), null);
        for (IAttributInfo info: attInfo.getInnereAttribute())
        {
          status.add(pruefeDatenDefinition(info, item, ignoreIsPflichtFeld));
        } 
      }
    }
    else
    {
      Assert.isTrue(item.isArray(), "An dieser Stelle muss der bergebene Datensatz fr \"" +
          attInfo.getName() + "\" ein Array liefern");
      status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Bearbeite Datenfeld " +
          attInfo.getName(), null);
      Array array = item.asArray();
      
      for (int i = 0; i < array.getLength(); i += 1)
      {
        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert " +
            (i + 1) + ". Element", null);
        if (!attInfo.hasInnereAttribute())
        {
          pruefeDatenDefinition(array.getItem(i), ignoreIsPflichtFeld ? false : 
            attInfo.isPflichtFeld(), meldung);
        }
        else
        {
          for (IAttributInfo info: attInfo.getInnereAttribute())
          {
            meldung.add(pruefeDatenDefinition(info, array.getItem(i), ignoreIsPflichtFeld));
          } 
        }
        status.add(meldung);
      }
    } 
      
    return status;
  }

  /**
   * Prft den Wert eines Attributes eines Datensatzes, ob dieser<br>
   * 1. Korrekt definiert ist<br>
   * 2. Bei Textattributen eine ggf. vorhandene Lngenbeschrnnkung<br>
   * 3. Der Wert des Attribut nicht leer ist, falls es als Pflichtfeld 
   * ausgewiesen ist<br>
   * Die Prfungen erfolgen rekursiv fr die nicht-atomaren Attributtypen Array 
   * und Attributliste
   * 
   * @param attInfo Attribut-Info des zu prfenden Attributs
   * @param daten Datensatz, der den zu prfenden Werte enthlt
   * @return Status, der das Ergebnis der Prfung enthlt
   */
  public static IStatus pruefeDatenDefinition(IAttributInfo attInfo, Data daten) 
  {
    return pruefeDatenDefinition(attInfo, daten, false);
  }
  
  /**
   * Prft ein Datum auf korrekte Definition (Textattribute ggf. auch auf korrelte Lnge), und optional 
   * auf Pflichtfeldeigenenschaft und schreibt das Ergebnis der Prfung in die bergebene Statusmeldung
   * 
   * @param item das zu prfende Datum
   * @param isPflichtFeld soll Pflichtfeldprfung stattfinden?
   * @param status Statusmeldung, in die das Erbenis geschrieben wird
   * 
   * @return <code>true</code>, falls die Prfung erfolgreich war, <code>false</code> sonst
   */
  private static boolean pruefeDatenDefinition(Data item, boolean isPflichtFeld, MultiStatus status)
  {
    if (!item.isDefined())
    {
      status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));
      return false;
    }
    else
    {
      status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
    }
    
    if (item.getAttributeType() instanceof StringAttributeType)
    {
      StringAttributeType attTyp = (StringAttributeType)item.getAttributeType();
      if (attTyp.isLengthLimited())
      {
        int maxLen = attTyp.getMaxLength();
        if (maxLen > 0)
        {
          String text = item.asTextValue().getText();
          if (text.length() > maxLen)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
                "Text ist lnger als die maximal erlaubte Lnge von " + maxLen + " Zeichen"));
            return false;
          }
          else
          {
            status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Textlnge: OK"));
          }
        }
      }
    }
          
    // Plichtfeld-Prfung:
    if (isPflichtFeld)
    { 
      String wert = item.asTextValue().getText();
      if ("".equals(wert) || (!(item.getAttributeType() instanceof StringAttributeType) && 
          "undefiniert".equals(wert) || "_Undefiniert_".equals(wert)))
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));
        return false;
      }
      else
      {
        status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
      }
    }
    return true;
  }
  
  /**
   * Prft die Werte der Attribute eines Datensatzes gem ATG-Info, ob diese<br>
   * 1. Korrekt definiert sind<br>
   * 2. Bei Textattributen eine ggf. vorhandene Lngenbeschrnnkung<br>
   * 3. Der Wert eines Attribut nicht leer ist, falls es als Pflichtfeld 
   * ausgewiesen ist<br>
   * Die Prfungen erfolgen rekursiv fr die nicht-atomaren Attributtypen Array 
   * und Attributliste
   * 
   * @param atgInfo ATG-Info, enthlt die zu prfenden Attribut-Infos
   * @param daten Datensatz, der die zu prfenden Werte enthlt
   * @return Status, der das Ergebnis der Prfung enthlt
   */
  public static IStatus pruefeDatenDefinition(AtgInfo atgInfo, Data daten)
  {
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
        "Prfe Daten der ATG " + atgInfo.getAtg(), null);
    
    for (String attribut: atgInfo.getAttribute())
    {
      IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
      result.add(pruefeDatenDefinition(attInfo, daten));
    }
    
    return result;
  }
  
  /**
   * Prft, ob ATG-Daten gendert werden sollen, die zu einer nicht-nderbaren
   * (OptionalConfigurationData) ATG gehren. Wenn derartige Daten gefunden werden, 
   * wird eine Meldungsbox mit entsprechendem Hinweis ausgegeben. 
   * 
   * @param ho das Hoerarchieobjekt, dessen Daten bernommen werden sollen
   * @param daten die zu bernehmenden Daten
   * @return <code>true</code> wenn keine nicht-nderbaren ATG-Daten in den 
   * zu bernehmenden Daten vorhanden sind oder wenn die Meldungsbox besttigt
   * wurde, <code>false</code> wenn die Meldungsbox abgebrochen wurde.
   */
  public boolean pruefeVorUebernehmen(HierarchieObjekt ho, HODaten daten)
  {
    Map<SystemObject, AttributeGroup> resultMap = new HashMap<SystemObject, AttributeGroup>();
    
    List<SystemObject> objektListe = daten.getSystemObjekte();

    for (int i = 0; i < objektListe.size(); i += 1)
    {
      SystemObject so = objektListe.get(i);
      
      if (ho.getSystemObjekte().contains(so))
      {
        SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
        
        // PID
        if (!so.getPid().equals(soDaten.getPid()))
        {
          // Wenn die PID gendert werden soll, wird sowieso ein neues Systemobjekt
          // erzeugt und alle ATG-Daten sind nderbar
          continue;
        }

        TypInfo typInfo = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid());
        
        // ATGs
        for (String atgPid: typInfo.getAtgs())
        {
          AttributeGroup atg = _dataModel.getAttributeGroup(atgPid);
          if (atg != null)
          {
            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_dataModel);
            
            //Data ist = so.getConfigurationData(atg);
            Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
            Data soll = soDaten.getAtgDaten(atgPid);
      
            if (soll != null)
            {
              if (ist == null || soDaten.isDatenGeaendert(atgPid))
              { 
                if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
                {
                  resultMap.put(so, atg);
                  continue;
                } 
              } 
            }
            else // Keine Daten fr diese ATG bergeben => evtl. vorhandene Daten sind zu lschen
            {
              if (ist != null)
              {
                if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
                {
                   resultMap.put(so, atg);
                   continue;
                }  
              }
            }
          }
        } 
        
        // Mengen
        for (String mengenName: typInfo.getDynMengen())
        {
          MutableSet menge = ((ConfigurationObject)so).getMutableSet(mengenName);
          
          if (menge != null)
          {
            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_dataModel);
            AttributeGroup atg = _dataModel.getAttributeGroup("atg.dynamischeMenge");
            
            Data ist = ((ConfigSystemObject)menge).getConfigurationData(atg, lookup);
            Data soll = soDaten.getDynMengenDaten(mengenName);
            
            if (soll != null)
            {
              if (ist == null || soDaten.isDatenGeaendert(mengenName))
              {  
                if (!HierarchieObjekt.isAttributgruppeAenderbar(menge, atg))
                {
                  resultMap.put(so, atg);
                  break;
                }
              }  
            }
            else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
            {
              if (ist != null)
              {
                if (!HierarchieObjekt.isAttributgruppeAenderbar(menge, atg))
                {
                  resultMap.put(so, atg);
                  break;
                }
              }
            }
          }
        }
      }     
    }  
    
    String soList = "";
    String atgList = "";
    
    for (SystemObject so: resultMap.keySet())
    {
      AttributeGroup atg = resultMap.get(so);
      
      if (atg != null)
      {  
        if (!soList.isEmpty())
        {
          soList += ", ";
        }
        soList += so.getPid();
        if (!atgList.isEmpty())
        {
          atgList += ", ";
        }
        atgList += atg.getPid();
      }
    }
    
    if (!resultMap.isEmpty())
    {
      String text = "bernehmen als nicht-nderbar definierter ATG-Daten";
      String message = "Daten der Attributgruppe(n) " + atgList + " sind durch \"" +
      AttributeGroupUsage.Usage.OptionalConfigurationData + "\" fr aktivierte " +
      "Systemobjekte im Datenmodell als nicht nderbar definiert." +
      "\nFolgende(s) Systemobjekt(e) werden gelscht und danach neu angelegt, wenn " + 
      "die Daten dennoch gendert werden sollen: " + soList +
      "\nSoll der Vorgang fortgesetzt werden?";
      
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
          SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
      msg.setText(text);
      msg.setMessage(message);
      int result = msg.open();
      if (result == SWT.CANCEL)
      {  
        return false;
      }
    }  
    
    return true;
  }
  
}
