/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.regeln;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;


/**
 * Schnittstelle fr Bildungsregeln<br>
 * Die  Schnittselle IBildungsregeln spezifiziert Methoden
 * 1. Zur Handhabung von Eingabefeldern und deren Werten, etwa zur Versorgung eines
 * Eingabedialogs. Die Werte der Eingabefelder werden zur Bildung von PID und Namen
 * herangezogen<br>
 * 2. Methoden zur Generierung von PID, Name und Attributgruppendaten der Systemobjekte
 * eines neu zu erzeugenden Hierarchieobjekts
 * 3. Methoden zum Datenaustausch etwa eines Eingabedialogs und der IBildungsregel-Instanz
 * 
 * @author Sans
 *
 */
public interface IBildungsRegeln extends BildungsRegelnBeobachter
{
  /**
   * Bestimmt die Hierarchie, die fr die Bildungsregeln benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Bildungsregeln benutzt werden kann
   */
  public IHierarchie getHierarchie();
  
  /**
   * Setzt die Hierarchie, die fr die Bildungsregeln benutzt werden kann
   * 
   * @param hierachie die Hierarchie, die fr die Bildungsregeln benutzt werden kann
   */
  public void setHierarchie(IHierarchie hierachie);
  
  /**
   * Bestimmt das Prfix fr die PID eines Systemobjekts
   * 
   * @param pidTypNeuesObjekt der Systemobjekttyp (als PID)
   * 
   * @return das PID-Prfix fr den bergebenen Typ
   */
  public String getPraefix(String pidTypNeuesObjekt);
  
  /**
   * Bestimmt die Namen der Eingabefelder, etwa fr einen Vorbelegungsdialog,
   * aus den Werten dieser Felder wird PID und Name fr Systemobjekte gebildet
   * 
   * @param hotNeuesObjekt der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * 
   * @return Liste der Namen der Eingabefelder
   */
  public String[] getEingabeFelder(IHierarchieObjektTyp hotNeuesObjekt);
  
  /**
   * Liefert eine Instanz vom Typ EingabeVorbelegung fr ein Eingabefeld
   * 
   * @param hotNeuesObjekt der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @param eingabeFeld der Name des Eingabefelds
   * @param verbundenesHO das verbundene Hieraechieobjekt, dies ist normalerweise das in der
   * Hierarchie bergeordnete HO
   * 
   * @return entsprechend den bergebenen Daten initialisierte Instanz vom Typ EingabeVorbelegung
   */
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, 
      HierarchieObjekt verbundenesHO);
  
  /**
   * Setzt den Wert eines Eingabefeldes zur Verwendung durch die Bildungsregeln
   * 
   * @param hotNeuesObjekt der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @param eingabeFeld der Name des Eingabefelds
   * @param wert der Wert, der in den Bildungsregeln verwendet werden soll
   */
  public void setWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, String wert);
  
  /**
   * Bestimmt den Wert eines Eingabefeldes, der aktuell in den Bildungsregeln benutzt wird
   * 
   * @param hotNeuesObjekt der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @param eingabeFeld der Name des Eingabefelds
   * 
   * @return den Wert des Eingabefelds, der in den Bildungsregeln verwendet wird
   */
  public String getWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld);
  
  /**
   * bernimmt alle Daten in die aktuellen Werte, aus denen PID und Name gebildet werden,
   * aus den Werten des bergeordeneten Objekts, sofern vorhanden.<br>
   * Wenn Werte nicht im bergeordneten Objekt vorhanden sind, werden sie aus dem 
   * Vorlageobjekt bestimmt.<br>
   * Der Aufruf dieser Methode hat die gleiche Wirkung wie der Aufruf des Bildungsregel-
   * Assistenten, die Werte werden aus dem Eltern-oder Volageobjekt genommen.<br>
   * Dieses Methode muss insbesondere fr das Kopieren und Verschieben von untergeordneten
   * Objekten korrekt und vollstndig implementiert werden
   * 
   * @param hotNeuesObjekt der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @param uebergeordnetesHO das dem neu zu erzeugenden HO bergeordnete HO
   * @param vorlageHO das HO, aus dessen Daten die Werte bestimmt werden, wenn sie fr
   * das bergeordnete HO nicht vorhanden sind
   * @return <code>true</code>, wenn die Werte erfolgreich gesetzt wurden und die Bildungregeln
   * angewendet werden knnen. Wenn <code>false</code> geliefert wird, bedeutet das, dass die 
   * Informationen nicht ausreichen, um die Bildungsregeln anzuwenden und der Bildungsregel-
   * Dialog aufgerufen werden sollte
   */
  public boolean setWerte (IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO, 
      HierarchieObjekt vorlageHO);
    
  /**
   * Gibt einen Informationstext aus, der aussagt, wie und aus welchen Eingabefeldern
   * die PID gebildet wird
   * 
   * @param hot der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @return Informationstext aus, der aussagt, wie und aus welchen Eingabefeldern
   * die PID gebildet wird
   */
  public String getPidRegelText(IHierarchieObjektTyp hot); 
  
  /**
   * Gibt einen Informationstext aus, der aussagt, wie und aus welchen Eingabefeldern
   * der Name eines Systemobjekts gebildet wird
   * 
   * @param hot der Hierachieobjekttyp des neu zu erzeugenden Hierarchieobjekts
   * @return Informationstext aus, der aussagt, wie und aus welchen Eingabefeldern
   * der Name eines Systemobjekts gebildet wird
   */
  public String getNameRegelText(IHierarchieObjektTyp hot);
  
  /**
   * Generiert aus den aktuellen Werten der Eingabefelder alle PID eines Hierarchieobjekts 
   * gemss den Bildungsregeln
   *  
   * @param hotNeuesObjekt der Hierarchieobjekttyp des neu zu erzeugenden Hierarchieobjektt
   * @param uebergeordnetesHO das dem neu zu erzeugenden HO bergeordnete HO
   * @return Liste der PID fr alle Systemobjekte des neu zu erzeugenden Hierarchieobjekts
   */
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO);
  
  /**
   * Generiert fr ein Systemobjekt aus den aktuellen Werten der Eingabefelder den Namen 
   * gemss den Bildungsregeln
   * 
   * @param objekt Systemobjekt, dessen Namen gebildet werden soll
   * @return den generierten Namen fr das bergebene Systemobjekt
   */
  public String getName(SystemObject objekt);
  
  /**
   * Generiert fr ein Systemobjekt eines Hierarchieobjekts die Daten eines Attributgruppen-
   * Datensatzes
   * 
   * @param ho das Hierarchieobjekt, fr dessen Systemobjekt Daten generiert werden sollen
   * @param objekt das Systemobjekt, fr das ein neuer Attributgruppen-Datensatz generiert
   * werden soll
   * @param atg die Attributgruppe des zu generierenden Datensatzes
   * @param daten Datensatz mit den aktuellen, ggf. zu berschreibende ATG-Daten als Vorlage. 
   * Dieser Datensatz ist nicht direkt zu beschreiben, sondern es ist eine schreibbare Kopie 
   * zu benutzen. Wird null bergeben, so wird ein mit Defaultwerten initialisierter Datensatz 
   * intern gebildet und benutzt
   * @param nurPflichtFelder Flag, das angibt, dass nur Pflichfelder generiert werden
   * @return einen neu generierten Attributgruppen-Datensatz gemss Bildungsregeln. Attribute,
   * fr die keine Bildungsregel existiert, bleiben unverndert bzw. werden mit ihrem 
   * Default-Wert belegt, wenn keinen aktueller Datensatz bergeben wurde
   */
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten, 
      boolean nurPflichtFelder);

  /**
   * Generiert fr ein Systemobjekt eines Hierarchieobjekts und einer Attributgruppe die Daten eines
   * Arrayelement-Datensatzes
   * 
   * @param ho
   *          das Hierarchieobjekt, fr dessen Systemobjekt Daten generiert werden sollen
   * @param objekt
   *          das Systemobjekt, fr das ein neuer Attributgruppen-Datensatz generiert werden soll
   * @param atg
   *          die Attributgruppe des zu generierenden Datensatzes
   * @param daten
   *          Array-Datensatz oder Datensatz mit den aktuellen ATG-Daten. Dieser Datensatz wird direkt beschrieben.
   * @param arrayName
   *          Name des Arrays
   * @param arrayIndex
   *          Index des Arrayelements oder -1 fr alle Array-Elemente
   * @param nurPflichtFelder
   *          Flag, das angibt, dass nur Pflichfelder generiert werden
   * @return bergebener Datensatz mit einem neu generierten Arrayelement-Datensatz gemss Bildungsregeln. Attribute,
   *         fr die keine Bildungsregel existiert, bleiben unverndert bzw. werden mit ihrem Default-Wert belegt, wenn
   *         kein aktueller Datensatz bergeben wurde
   */
  public Data getDatenArrayElement(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten, String arrayName, int arrayIndex,
      boolean nurPflichtFelder);
  
  /**
   * Meldet die bergebene BildungsRegelnBeobachter-Instanz auf intern von dieser IBildungsregeln-
   * Instanz durchgefhrte nderungen an, etwa, wenn beim Setzen eines Wertes ein oder mehrere 
   * anderere Werte zu ndern sind. Wenn dies nie der Fall ist, wird die Methode mit leerem 
   * Body implementiert.
   *  
   * @param beobachter zu informierende BildungsRegelnBeobachter-Instanz
   */
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter);
  
  /**
   * Informiert alle angemeldeten BildungsRegelnBeobachter-Instanzen ber eine intern durchgefhrte
   * nderung eines Wertes. Wenn es keine intern durchgefhrten nderungen gibt, wird die Methode mit 
   * leerem Body implementiert
   * 
   * @param feldName Feldname, wo die interne nderung durchgefhrt wurde
   * @param wert neuer Wert, der durch die interne nderung entstanden ist
   */
  public void alleBeobachterInformieren(String feldName, String wert);
  
  /**
   * Meldet eine angemeldete BildungsRegelnBeobachter-Instanz ab. Wenn es keine intern durchgefhrten 
   * nderungen gibt, wird die Methode mit leerem Body implementiert
   * 
   * @param beobachter abzumeldende BildungsRegelnBeobachter-Instanz
   */
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter);
  
  /**
   * Die IBildungsregeln-Instanz wird ber externe nderungen am Wert eines Eingabefelds
   * informiert, etwa, wenn im Eingabedialog ein Wert gendert wird, und aufgrund dieser
   * nderung eine interne nderung an anderen Werten innerhalb dieser Instanz erfolgen muss.
   * Wenn nicht auf externe nderungen reagiert werden muss, z. B. wenn aufgrund von externen
   * nderungen keine anderen Werte gendert werden mssen, wird die Methode mit leerem 
   * Body implementiert.
   * 
   * @param feldName der Name des Eingabefeldes
   * @param wert der genderte Wert des Eingabefeldes
   */
  public void wertGeaendert(String feldName, String wert);
}
